import { computed, type Ref, ref } from 'vue'
import type { EChartsOption } from 'echarts/types/dist/shared'
import axiosInstance from '@/utils/axios/axiosInstance.ts'
import type { ResponseInfo } from '@/types/res.ts'
import type { QueryInfo } from '@/types/table.ts'
import { generateUniqueColors } from '@/utils/common'
import PieBorderRadius from '@/components/echarts/PieBorderRadius.vue'
type MChartType = InstanceType<typeof PieBorderRadius>
interface UserBehaviorChartItem {
  name: string
  value: number
}

interface ChartInfo {
  PieChart: UserBehaviorChartItem[]
  BarChart: {
    labelList: string[]
    valueList: number[]
  }
}

export function useUserBehaviorChart(
  url: string,
  queryFormData: Ref<any>,
  filterInfo: Array<QueryInfo>,
  chartNeedFields: Array<string>,
  isPie: Ref<boolean>,
  chartInstance: Ref<MChartType | null>,
  isLog: Ref<boolean>
) {
  const chartInfo = ref<ChartInfo>({
    PieChart: [],
    BarChart: {
      labelList: [],
      valueList: []
    }
  })

  const updateChartData = async () => {
    try {
      if (chartNeedFields.length > 0) {
        const hasFilter = chartNeedFields.every((item) => {
          return queryFormData.value[item] === ''
        })
        if (hasFilter) {
          const tip = filterInfo
            .filter((item) => {
              return chartNeedFields.includes(item.name as any)
            })
            .map((item) => {
              return item.label
            })
            .join(',')
          ElMessage.warning(`${tip}请至少输入一个筛选条件`)
          return
        }
      }

      if (chartInstance.value) {
        chartInstance.value.startLoading()
      }

      const params = {} as any
      Object.assign(params, queryFormData.value)
      if (queryFormData.value.createTime) {
        params.createTime = queryFormData.value.createTime
          .map((item: any) => {
            return new Date(item).getTime().toString()
          })
          .join(',')
      }

      const res = (await axiosInstance.post(url, params)) as ResponseInfo
      if (res.code !== 0) {
        ElMessage.error('获取数据失败')
        return
      }

      const data = res.data as Array<{
        name: string
        count: number
      }> | null

      if (!data) {
        chartInfo.value.PieChart = []
        chartInfo.value.BarChart = {
          labelList: [],
          valueList: []
        }
      } else {
        const isAllZero = data.every((item) => item.count === 0)
        if (isAllZero) {
          chartInfo.value.PieChart = []
          chartInfo.value.BarChart = {
            labelList: [],
            valueList: []
          }
          return
        }

        chartInfo.value.PieChart = data.map((item) => ({
          name: item.name,
          value: item.count
        }))
        chartInfo.value.BarChart = {
          labelList: data.map((item) => item.name),
          valueList: data.map((item) => item.count)
        }
      }
    } catch (err) {
      console.log(err)
    } finally {
      console.log(chartInstance)
      if (chartInstance.value) {
        chartInstance.value.stopLoading()
      }
    }
  }

  const pieChartOptions = computed<EChartsOption>(() => {
    const colors = generateUniqueColors(chartInfo.value.PieChart.length)
    const pieData = chartInfo.value.PieChart
    const seriesData: {
      [key: string]: any[]
    } = {}
    pieData.forEach((item) => {
      const name = item.name.split('&&')[0]
      if (!seriesData[name]) {
        seriesData[name] = []
      }
      seriesData[name].push(item)
    })
    const resultSeries: any[] = []
    let count = 1
    for (const [k, v] of Object.entries(seriesData)) {
      console.log(`${count * 5}% `, `${count * 10}% `)
      resultSeries.push({
        name: k,
        type: 'pie',
        radius: [`${count * 10}% `, `${(count + 1) * 10}% `],
        center: ['50%', '50%'],
        avoidLabelOverlap: false,
        itemStyle: {
          borderRadius: 10,
          borderColor: '#fff',
          borderWidth: 1
        },
        label: {
          show: false,
          position: 'center'
        },
        emphasis: {
          label: {
            show: true,
            fontSize: 20,
            fontWeight: 'bold'
          }
        },
        labelLine: {
          show: false
        },
        data: v
      })
      count++
    }
    console.log(resultSeries)
    return {
      color: colors,
      // tooltip: {
      //   trigger: 'item',
      //   formatter: (params: any) => {
      //     const data = chartInfo.value.PieChart[params.dataIndex]
      //     return `${data.name}<br/>
      //             数量: ${data.value}`
      //   }
      // },
      tooltip: {
        trigger: 'item',
        formatter: '{a} <br/>{b}: {c} ({d})'
      },
      legend: {
        type: 'scroll',
        orient: 'vertical',
        right: 10,
        top: 20,
        bottom: 20
        // top: '8%',
        // left: 'center',
        // itemGap: 35,
        // textStyle: {
        //   fontSize: 14
        // },
        // padding: [0, 50]
      },
      series: resultSeries
    }
  })

  const barChartOptions = computed<EChartsOption>(() => {
    const barChartInfo = chartInfo.value.BarChart
    const yType: any = isLog.value ? 'log' : 'value'
    return {
      xAxis: {
        type: 'category',
        data: barChartInfo.labelList,
        axisLabel: {
          interval: 30,
          // rotate: 30,
          // width: 10, // 增加标签宽度
          overflow: 'truncate' // 过长时显示省略号
        }
      },
      yAxis: {
        type: yType,
        min: 1,
        axisLabel: {
          formatter: (value: number) => value.toLocaleString()
        }
      },
      dataZoom: [
        {
          type: 'slider',
          show: true,
          start: 0,
          end: 100,
          handleSize: 8
        },
        {
          type: 'inside',
          start: 0,
          end: 100
        },
        {
          type: 'slider',
          show: true,
          yAxisIndex: 0,
          filterMode: 'empty',
          width: 12,
          height: '70%',
          handleSize: 8,
          showDataShadow: false,
          left: '93%'
        }
      ],
      color: '#9FE080',
      tooltip: {
        trigger: 'axis',
        axisPointer: {
          type: 'shadow'
        },
        formatter: (params: any) => {
          const data = chartInfo.value.PieChart[params[0].dataIndex]
          return `${data.name}<br/>
                  数量: ${data.value}`
        }
      },
      grid: {
        top: '15%',
        left: '3%',
        right: '4%',
        bottom: '15%',
        containLabel: true
      },
      series: [
        {
          name: '用户行为',
          data: barChartInfo.valueList,
          type: 'bar',
          barWidth: '10%',
          label: {
            show: true,
            position: 'top',
            formatter: '{c}'
          },
          itemStyle: {
            // color: (params) => {
            //   return colors[params.dataIndex]
            // }
          }
        }
      ]
    }
  })

  const chartOptions = computed(() => {
    const pieChartInfo = chartInfo.value.PieChart.length ?? null
    const barChartInfo = chartInfo.value.BarChart.valueList.length ?? null
    if (!pieChartInfo && !barChartInfo) return null
    return isPie.value ? pieChartOptions.value : barChartOptions.value
  })

  return {
    updateChartData,
    chartOptions
  }
}
